#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QDrag>
#include <QMimeData>
#include <QGraphicsSceneDragDropEvent>
#include <QApplication>

void Ellipse::dragEnterEvent(QGraphicsSceneDragDropEvent *event)
{
    if (event->mimeData()->hasText() || event->mimeData()->hasColor())
       event->setAccepted(true);
}

void Ellipse::dropEvent(QGraphicsSceneDragDropEvent *event)
{
    const QMimeData * data = event->mimeData();
    if (data->hasColor())
    {
       QColor c = qvariant_cast <QColor> ( data->colorData() );
       setBrush (c);
    }
    if (data->hasText())
    {
        QString s = data->text ();
        QGraphicsTextItem * item = new QGraphicsTextItem;
        item->setDefaultTextColor(QColor ("lime"));
        item->setFont (QFont ("Sans Serif", 20));
        item->setPlainText (s);
        item->setFlags (QGraphicsItem::ItemIsMovable | QGraphicsItem::ItemIsSelectable);
        item->setParentItem (this);
    }

}

Ellipse::Ellipse ()
{
    setAcceptDrops(true);
    setPen (QColor ("blue"));

    /*
    // QLinearGradient d (0, 0, 200, 100);
    QRadialGradient d (100, 100, 100, 100, 50);
    d.setColorAt(0.0, QColor ("yellow"));
    d.setColorAt (1.0, QColor ("blue"));
    QBrush b (d);
    setBrush (b);
    */
    setBrush (QColor ("yellow"));
    setRect (0, 0, 200, 100);
    setFlags (QGraphicsItem::ItemIsMovable | QGraphicsItem::ItemIsSelectable);
}

QPixmap image (int size, QColor color)
{
   QPixmap p (size, size);
   p.fill (Qt::transparent);
   QPainter t (&p);
   t.setPen (color.lighter());
   t.setBrush (color);
   t.drawEllipse (1, 1, size-2, size-2);
   return p;
}

QIcon mark (QColor color)
{
    return QIcon (image (32, color));
}

ColorButton::ColorButton (QColor c)
   : color (c)
{
   setIcon (mark (color));
}

void ColorButton::mousePressEvent(QMouseEvent *event)
{
    QMimeData * data = new QMimeData;
    data->setColorData (color);
    QDrag * drag = new QDrag (this);
    drag->setMimeData(data);
    drag->start ();
}

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    QStringList names;
    names << "red" << "green" << "blue" << "yellow" << "orange";
    int x = 0;
    for (QString name : names)
    {
        QColor color (name);
        ColorButton * b = new ColorButton (color);
        b->setToolTip (name);
        b->setParent (ui->colorTab);
        b->move (x, 0);
        x += 36;
    }

    QPixmap p (24, 24);
    p.fill (Qt::transparent);
    QPainter t (&p);
    // t.fillRect(0, 0, 24, 24, QColor (255,255,255));
    t.setPen (QColor ("blue"));
    t.setBrush (QColor ("orange"));
    t.drawEllipse(0, 0, 20, 20);
    QIcon icon (p);
    // ui->toolButton->setIcon(icon);

    QGraphicsScene * scene = new QGraphicsScene;
    ui->graphicsView->setScene (scene);
    scene->addLine (0, 0, 100, 100, QColor ("red"));

    scene->addItem (new Ellipse);

    for (int i = 1; i <= 3; i++)
    {
        QGraphicsRectItem * r = new QGraphicsRectItem;
        r->setRect (0, 0, 40, 40);
        r->setPos (40 + 10*i, 40+10*i);
        r->setPen (QColor ("blue"));
        r->setBrush(QColor::fromHsv (60*i, 255, 255));
        r->setToolTip ("Muj obdelnik " + QString::number (i));
        r->setFlags (QGraphicsItem::ItemIsMovable | QGraphicsItem::ItemIsSelectable);
        scene->addItem (r);
    }
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::on_action_Quit_triggered()
{
    close ();
}

int main(int argc, char *argv[])
{
    QApplication a(argc, argv);
    MainWindow w;
    w.show();

    return a.exec();
}
